
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "plusscript.h"

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>

void print(char *msg)
{
	printf("%s\n", msg);
}
void printint(int value)
{
	printf("%d\n", value);
}
void printfloat(float value)
{
	printf("%f\n", value);
}

const char *tests[2] = {
"tests/if.ps",
"tests/float.ps"
};
int teststatus[2];
int testcount = 2;

typedef int (*TESTFUNCTION)(void);

int main(int argc, char **argv)
{
	FILE *log = fopen("plusscript.log", "wt");
	set_logging_output(log);
	int i;
	for (i = 0; i < testcount; i++)
	{
		printf("======================================================\n");
		printf("Running test \"%s\".\n", tests[i]);
		printf("======================================================\n");
		// Create VM
		fs_vm_t *vm = create_vm();
		int argtype = VAR_TYPE_USERPTR;
		vm_register_function(vm, (void*)print, "print", 1, &argtype);
		argtype = VAR_TYPE_INT;
		vm_register_function(vm, (void*)printint, "printint", 1, &argtype);
		argtype = VAR_TYPE_FLOAT;
		vm_register_function(vm, (void*)printfloat, "printfloat", 1, &argtype);
	
		module_t *testmod = parse_source(vm, tests[i]);
		//display_bytecode(testmod);
	
		create_stubs(vm, testmod);
		vm_add_module(vm, testmod);
	
		int (*testentry)(void);
		testentry = (TESTFUNCTION)vm_get_function_address(vm, "runtest");
		if (!testentry)
		{
			printf("Entry point not found.\n");
			teststatus[i] = 0;
		}
		else
		{
			teststatus[i] = testentry();
		}
	
		delete_module(vm, testmod);
		delete_vm(vm);
	}
	
	printf("\n");
	for (i = 0; i < testcount; i++)
	{
		printf("Test %d (%s): \t", i, tests[i]);
		if (teststatus[i])
		{
			printf("success.\n");
		}
		else
		{
			printf("failure.\n");
		}
	}
	
	if (log) fclose(log);
	
	return 0;
}

